@@ -22,7 +22,6 @@ from neutron_lib.api import extensions
 from neutron_lib.exceptions import firewall_v2 as f_exc
 from neutron_lib.services import base as service_base
 from oslo_config import cfg
-import six
 
 from neutron_fwaas._i18n import _
 from neutron_fwaas.common import fwaas_constants
@@ -172,7 +171,22 @@ default_fwg_rules_opts = [
                 help=_('Firewall group rule enabled. '
                        'Default is True.')),
 ]
+firewall_quota_opts = [
+    cfg.IntOpt('quota_firewall_group',
+               default=10,
+               help=_('Number of firewall groups allowed per tenant. '
+                      'A negative value means unlimited.')),
+    cfg.IntOpt('quota_firewall_policy',
+               default=10,
+               help=_('Number of firewall policies allowed per tenant. '
+                      'A negative value means unlimited.')),
+    cfg.IntOpt('quota_firewall_rule',
+               default=100,
+               help=_('Number of firewall rules allowed per tenant. '
+                      'A negative value means unlimited.')),
+]
 cfg.CONF.register_opts(default_fwg_rules_opts, 'default_fwg_rules')
+cfg.CONF.register_opts(firewall_quota_opts, 'QUOTAS')
 
 
 # TODO(Reedip): Remove the convert_to functionality after bug1706061 is fixed.
@@ -206,8 +220,9 @@ class Firewall_v2(extensions.APIExtensionDescriptor):
         return Firewallv2PluginBase
 
 
-@six.add_metaclass(abc.ABCMeta)
-class Firewallv2PluginBase(service_base.ServicePluginBase):
+class Firewallv2PluginBase(
+    service_base.ServicePluginBase,
+    metaclass=abc.ABCMeta):
 
     def get_plugin_type(self):
         return fwaas_constants.FIREWALL_V2
