@@ -23,6 +23,7 @@ from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
 from neutron_lib import constants as nl_constants
+from neutron_lib.db import api as db_api
 from neutron_lib.exceptions import firewall_v2 as f_exc
 from neutron_lib.plugins import constants as plugin_const
 from neutron_lib.plugins import directory
@@ -264,21 +265,21 @@ class FirewallPluginV2(Firewallv2PluginBase):
                 firewall_rule_id=rule_info['firewall_rule_id'])
 
     @registry.receives(resources.PORT, [events.AFTER_UPDATE])
-    def handle_update_port(self, resource, event, trigger, **kwargs):
-        updated_port = kwargs['port']
+    def handle_update_port(self, resource, event, trigger, payload):
+        context = payload.context
+        original_port = payload.states[0]
+        updated_port = payload.states[1]
         if not updated_port['device_owner'].startswith(
                 nl_constants.DEVICE_OWNER_COMPUTE_PREFIX):
             return
 
-        if (kwargs.get('original_port')[pb_def.VIF_TYPE] !=
-                pb_def.VIF_TYPE_UNBOUND):
+        if (original_port[pb_def.VIF_TYPE] != pb_def.VIF_TYPE_UNBOUND):
             # Checking newly vm port binding allows us to avoid call to DB
             # when a port update_event like restart, setting name, etc...
             # Moreover, that will help us in case of tenant admin wants to
             # only attach security group to vm port.
             return
 
-        context = kwargs['context']
         port_id = updated_port['id']
         # Check port is supported by firewall driver
         if not self._is_supported_l2_port(context, port_id):
@@ -312,6 +313,7 @@ class FirewallPluginV2(Firewallv2PluginBase):
 
     # Firewall Group
     @log_helpers.log_method_call
+    @db_api.CONTEXT_WRITER
     def create_firewall_group(self, context, firewall_group):
         firewall_group = firewall_group['firewall_group']
         ports = firewall_group.get('ports', [])
@@ -328,6 +330,7 @@ class FirewallPluginV2(Firewallv2PluginBase):
         return self.driver.create_firewall_group(context, firewall_group)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_WRITER
     def delete_firewall_group(self, context, id):
         # if no such group exists -> don't raise an exception according to
         # 80fe2ba1, return None
@@ -342,14 +345,17 @@ class FirewallPluginV2(Firewallv2PluginBase):
         self.driver.delete_firewall_group(context, id)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_READER
     def get_firewall_group(self, context, id, fields=None):
         return self.driver.get_firewall_group(context, id, fields=fields)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_WRITER
     def get_firewall_groups(self, context, filters=None, fields=None):
         return self.driver.get_firewall_groups(context, filters, fields)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_WRITER
     def update_firewall_group(self, context, id, firewall_group):
         firewall_group = firewall_group['firewall_group']
         ports = firewall_group.get('ports', [])
@@ -370,23 +376,28 @@ class FirewallPluginV2(Firewallv2PluginBase):
 
     # Firewall Policy
     @log_helpers.log_method_call
+    @db_api.CONTEXT_WRITER
     def create_firewall_policy(self, context, firewall_policy):
         firewall_policy = firewall_policy['firewall_policy']
         return self.driver.create_firewall_policy(context, firewall_policy)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_WRITER
     def delete_firewall_policy(self, context, id):
         self.driver.delete_firewall_policy(context, id)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_READER
     def get_firewall_policy(self, context, id, fields=None):
         return self.driver.get_firewall_policy(context, id, fields)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_READER
     def get_firewall_policies(self, context, filters=None, fields=None):
         return self.driver.get_firewall_policies(context, filters, fields)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_WRITER
     def update_firewall_policy(self, context, id, firewall_policy):
         firewall_policy = firewall_policy['firewall_policy']
         self._ensure_update_firewall_policy(context, id)
@@ -394,35 +405,42 @@ class FirewallPluginV2(Firewallv2PluginBase):
 
     # Firewall Rule
     @log_helpers.log_method_call
+    @db_api.CONTEXT_WRITER
     def create_firewall_rule(self, context, firewall_rule):
         firewall_rule = firewall_rule['firewall_rule']
         return self.driver.create_firewall_rule(context, firewall_rule)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_WRITER
     def delete_firewall_rule(self, context, id):
         self.driver.delete_firewall_rule(context, id)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_READER
     def get_firewall_rule(self, context, id, fields=None):
         return self.driver.get_firewall_rule(context, id, fields)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_READER
     def get_firewall_rules(self, context, filters=None, fields=None):
         return self.driver.get_firewall_rules(context, filters, fields)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_WRITER
     def update_firewall_rule(self, context, id, firewall_rule):
         firewall_rule = firewall_rule['firewall_rule']
         self._ensure_update_firewall_rule(context, id)
         return self.driver.update_firewall_rule(context, id, firewall_rule)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_WRITER
     def insert_rule(self, context, policy_id, rule_info):
         self._ensure_update_firewall_policy(context, policy_id)
         self._validate_insert_remove_rule_request(rule_info)
         return self.driver.insert_rule(context, policy_id, rule_info)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_WRITER
     def remove_rule(self, context, policy_id, rule_info):
         self._ensure_update_firewall_policy(context, policy_id)
         self._validate_insert_remove_rule_request(rule_info)
