@@ -14,7 +14,6 @@
 #    under the License.
 
 import mock
-import six
 
 from neutron import extensions as neutron_extensions
 from neutron.tests.unit.extensions import test_l3
@@ -92,12 +91,16 @@ class TestAgentDriver(test_fwaas_plugin_v2.FirewallPluginV2TestCase,
                       test_l3.L3NatTestCaseMixin):
 
     def setUp(self):
-        self.agentapi_del_fw_p = mock.patch(
+        self._mock_agentapi_del_fw_p = mock.patch(
             DELETEFW_PATH, create=True,
             new=FakeAgentApi().delete_firewall_group,
-        ).start()
-        mock.patch.object(agents.neutron_rpc, 'get_client').start()
-        mock.patch.object(agents.neutron_rpc, 'Connection').start()
+        )
+        self.agentapi_del_fw_p = self._mock_agentapi_del_fw_p.start()
+        self.addCleanup(self._mock_agentapi_del_fw_p.stop)
+        self._mock_get_client = mock.patch.object(agents.n_rpc, 'get_client')
+        self._mock_get_client.start()
+        self.addCleanup(self._mock_get_client.stop)
+        mock.patch.object(agents.n_rpc, 'Connection').start()
 
         l3_plugin_str = ('neutron.tests.unit.extensions.test_l3.'
                          'TestL3NatServicePlugin')
@@ -522,7 +525,7 @@ class TestAgentDriver(test_fwaas_plugin_v2.FirewallPluginV2TestCase,
                             fwr['firewall_rule']['id'])
                         res = self.deserialize(self.fmt,
                                                req.get_response(self.ext_api))
-                        for k, v in six.iteritems(attrs):
+                        for k, v in attrs.items():
                             self.assertEqual(v, res['firewall_rule'][k])
 
     def test_update_firewall_rule_on_pending_create_fwg(self):
